#!/usr/bin/env python2

import sys
import re
import numpy as np

import map as mp
import helper as hlp
import mgmp_solver as mgmp

import matplotlib.pyplot as plt

#instantiation of map and planner
map = mp.Map()
solver = mgmp.MGMP_solver()

################################################
# Testing
################################################
#maze name, (size_x, size_y) in vrep, (offset_x, offset_y) for coordinates transformation, voxel_size, start_position in the map, goal_position in the map, execution flag
test = ("./mazes/maze1.png",(10,8.6),(5,4.3),0.1,"./mazes/maze1.config")


#read map
map.from_file(test[0],test[1],test[2],test[3])

#read config with goal positions
goals = []
with open(test[4]) as fp:
	for line in fp:
		x, y, radius = line.split()
		goals.append((int(x), int(y), float(radius)))

#create neighborhoods (only for TSPN -- with neighborhoods)
neighbors={}
for i in range(0, len(goals)):
	g = goals[i]
	ne = map.target_neighbors((g[0], g[1]), g[2])
	#print(ne)
	neighbors[i] = ne

######################################
# plot arena and goals (and their neighborhoods)
######################################
#plot map
hlp.plot_map(map.get_grid())
#plot neighborhoods
for i in range(0, len(goals)):
	ne = neighbors[i]
	#print(ne)
	hlp.plot_points(ne, 'go')
#plot goals
hlp.plot_points(goals, 'ro')
######################################
#tour planning
######################################
#path = solver.plan_tour_etsp(map, goals)
path = solver.plan_tour_obstacles(map, goals, "E8")
#path = solver.plan_tour_obstacles_neighborhood(map, goals)
hlp.plot_points(path, 'r')

# print results with active waiting
while (1):
	hlp.pause(1)

