#!/usr/bin/env python2

import sys
import re
import numpy as np
import random

import map as mp
import helper as hlp
import mgmp_solver as mgmp

import matplotlib.pyplot as plt

#instantiation of map and planner
map = mp.Map()
solver = mgmp.MGMP_solver()

################################################
# Testing
################################################
#maze name, (size_x, size_y) in vrep, (offset_x, offset_y) for coordinates transformation, voxel_size, start_position in the map, goal_position in the map, execution flag
#test = ("./mazes/maze1.png",(10,8.6),(5,4.3),0.1,"./mazes/maze1.config")
test = ("./mazes/pacman.png",(19,21),(0,0),1,"./mazes/pacman.config")


#read map
map.from_file(test[0],test[1],test[2],test[3])

#read config with goal positions
goals = []
with open(test[4]) as fp:
	for line in fp:
		(x, y, radius) = line.split()
		goals.append((int(x), int(y), float(radius)))

start = goals[0]
end = goals[len(goals)-1]

# use only sub-set of goals (good for testing)
N = 200
# TODO - enable this
open_loop = False

# remove start and goal
if open_loop:
	goals = list(filter(lambda x: x != start and x != end, goals))
	
random.shuffle(goals)
print(goals)

if open_loop:
	goals = [goals[i] for i in range(N-2)]
else:
	goals = [goals[i] for i in range(N)]

# start and end are the first two goals
if open_loop:
	goals = [start, end] + goals

######################################
# plot arena and goals (and their neighborhoods)
######################################
#plot map
hlp.plot_map(map.get_grid())
hlp.plot_points(goals, 'ro')


start_idx = None
end_idx = None
if open_loop:
	start_idx = 0
	end_idx = 1

######################################
#tour planning
######################################
path = solver.plan_tour_etsp(map, goals, start_idx, end_idx)
#path = solver.plan_tour_obstacles(map, goals, "E4", start_idx, end_idx)
#path = solver.plan_tour_obstacles_Floyd_Warshall(map, goals, "E4", start_idx, end_idx)

######################################
# plot result
######################################
hlp.plot_points(path, 'r')
hlp.pause(1)

# simulate path
for i in range(0, len(path)):
	hlp.plot_map(map.get_grid())
	hlp.plot_points(goals, 'ro')
	hlp.plot_points(path, 'r')
	hlp.plot_points(path[:i], 'yo')
	hlp.plot_points([path[i]], 'go')
	hlp.pause(0.01)
	#hlp.save_figure('img/iter-{:04d}.png'.format(i))

# active waiting for end
while (1):
	hlp.pause(1)

