# -*- coding: utf-8 -*-
"""
@author: P.Vana & P.Cizek
"""
import sys
import os
import numpy as np

import math

import map as mp
import planner as pl

import matplotlib.pyplot as plt

from invoke_LKH import *

class MGMP_solver:
	def __init__(self):
		pass
	
	def select_tour(self, start_idx = None, end_idx = None):
		n = len(self.distances)
		
		if start_idx != None and end_idx != None:
			# TODO - insert your code here
			print("TODO")
		
		fname_tsp = "problem"
		user_comment = "a comment by the user"
		writeTSPLIBfile_FE(fname_tsp,
					 self.distances,user_comment)
		run_LKHsolver_cmd(fname_tsp)
		sequence = read_LKHresult_cmd(fname_tsp)
		
		if start_idx != None and end_idx != None:
			# TODO - reconstruct only the open-loop
			print("TODO")

		else:
			path = []
			for a in range(0,n):
				b = (a+1) % n
				a_idx = sequence[a]
				b_idx = sequence[b]
				actual_path = self.paths[(a_idx,b_idx)];
				path = path + actual_path
		
		return path
		
	
	def plan_tour_etsp(self, map, goals, start_idx = None, end_idx = None):
		n = len(goals)
		self.distances = np.zeros((n,n))	
		self.paths = {}

		planner = pl.Planner();

		# find path between each pair of goals (a,b)
		for a in range(0,n): 
			for b in range(0,n):
				g1 = goals[a]
				g2 = goals[b]
				if a != b:
					# find the shortest path
					sh_path = [g1[:2]] + [g2[:2]]
					# store distance
					self.distances[a][b] = map.cost_euclidean_path(sh_path) 
					# store path
					self.paths[(a,b)] = sh_path

		return self.select_tour(start_idx, end_idx)
	
	def plan_tour_obstacles(self, map, goals, neigh,  start_idx = None, end_idx = None):	
		n = len(goals)
		self.distances = np.zeros((n,n))	
		self.paths = {}

		planner = pl.Planner();

		# find path between each pair of goals (a,b)
		for a in range(0,n): 
			print('Shortest paths from city {0}'.format(a))
			for b in range(0,n):
				if a != b:
					g1 = goals[a]
					g2 = goals[b]
				
					# find the shortest path
					sh_path = planner.plan(map, g1[:2], g2[:2], neigh)
					# store distance
					self.distances[a][b] = map.cost_euclidean_path(sh_path)  
					# store path
					self.paths[(a,b)] = sh_path

		return self.select_tour(start_idx, end_idx)
	
	def plan_tour_obstacles_Floyd_Warshall(self, map, goals, neigh,  start_idx = None, end_idx = None):	
		N = len(goals)
		self.distances = np.zeros((N,N))
		self.paths = {}

		planner = pl.Planner();
		
		nodes = map.all_nodes()
		
		# look-up tables for converting node and its index
		node_2_idx = {}
		idx_2_node = {}
		for n in enumerate(nodes):
			node_2_idx[n[1]] = n[0]
			idx_2_node[n[0]] = n[1]
			
		goal_idx = [node_2_idx[(int(g[0]), int(g[1]))] for g in goals]
		print(goal_idx)
		
		# edges -- (idx1, idx2) : distance
		edges = {}
		for n1 in nodes:
			if(neigh == "E4"):
				neighs = map.neighbors4(n1)
			else:
				neighs = map.neighbors8(n1)
			for n2 in neighs:
				idx1 = node_2_idx[n1]
				idx2 = node_2_idx[n2]
				edges[(idx1, idx2)] = map.cost_euclidean(n1,n2)
		#print(edges)
		
		M = len(nodes)
		node_dst = np.ones((M,M)) * float("inf")
		node_prev = -np.ones((M,M), dtype=int)
		for i in range(M): node_dst[i,i] = 0
		
		# Floyd Warshall algorithm
		# TODO - implement here
		
		improving = True
		while improving:
			improving = False


		# reconstruct paths between goals from 'node_dst' matrix
		for a in range(0,N): 
			for b in range(0,N):
				if a != b:
					g1 = goals[a]
					g2 = goals[b]
					idx1 = goal_idx[a]
					idx2 = goal_idx[b]
					self.distances[a][b] = node_dst[idx1, idx2]
					# TODO

		return self.select_tour(start_idx, end_idx)
	
	def reconstruct_path_Floyd_Warshall(self, n1, n2, node_prev, idx_2_node):
		#print('{} {}'.format(n1, n2))
		# TODO
		sequence = [n2]
		path = []
		return path
	
	def plan_tour_obstacles_neighborhood(self, map, goals, neighs):	
		n = len(goals)
		self.distances = np.zeros((n,n))	
		self.paths = {}

		planner = pl.Planner();

		# find path between each pair of goals (a,b)
		for a in range(0,n): 
			for b in range(0,n):
				g1 = goals[a]
				g2 = goals[b]
				if a != b:
					# find the shortest path
					sh_path = planner.plan(map, g1[:2], g2[:2], neigh)
					# store distance
					self.distances[a][b] = map.cost_euclidean_path(sh_path) 
					# store path
					self.paths[(a,b)] = sh_path

		return self.select_tour()
	 
	 
	
